#!/bin/ksh

export TPDPWFILE=/opt/3PAR/inform_cli_3.1.1/bin/up
TIMESTEP=60
CLI=/opt/3PAR/inform_cli_3.1.1/bin/cli
LOCK=/tmp/.3par.lock
CFILE=/tmp/.3par.cpu
VFILE=/tmp/.3par.vols
DFILE=/tmp/.3par.ports.disk
HFILE=/tmp/.3par.ports.host
SFILE=/tmp/.3par.ports.*
FCFILE=/tmp/.3par.freespace.fc
NLFILE=/tmp/.3par.freespace.nl
SSDFILE=/tmp/.3par.freespace.ssd
EXE=$0
HOST=$1
COM=$2
VAL=$3
NSP=$4
NSPC=`print $NSP | awk '{print substr($1,1,1)":"substr($1,2,1)":"substr($1,3,1)}'`
VOL=$4

function usage
{
	print "Usage: $EXE HOSTNAME index|num_indexes|query|get|diskPorts|hostPorts|cpu|cpuIntr|cpuCtxt|vvindex|vvnum_indexes|vvquery|vvget|freeSpace"
	exit 1
}

if [[ $2 == "" ]]; then usage; fi

LDATE=`stat -L --format %Y $LOCK 2>/dev/null`
if [[ $LDATE != "" ]] && (( ( LDATE + ( 2 * TIMESTEP ) ) < `date +%s` )) then
	rm $LOCK 
fi

while [[ -f $LOCK ]]
do
	sleep 1
done

if [[ ! -f $DFILE ]] || (( `stat -L --format %Y $DFILE` < ( `date +%s` - $TIMESTEP + 15 ) )) then
	if [[ ! -f $LOCK ]] then
		touch $LOCK
		$CLI -sys $HOST statport  -d 5 -iter 1 -rw -disk | awk '{print $0" DiskPort"}' > $DFILE &
		$CLI -sys $HOST statport  -d 5 -iter 1 -rw -host | awk '{print $0" HostPort"}' > $HFILE &
		$CLI -sys $HOST statvv    -d 5 -iter 1 -rw  > $VFILE &
		$CLI -sys $HOST statcpu   -d 5 -iter 1  -t  > $CFILE &
		$CLI -sys $HOST showsys -space -devtype FC  > $FCFILE &
		$CLI -sys $HOST showsys -space -devtype NL  > $NLFILE &
		$CLI -sys $HOST showsys -space -devtype SSD > $SSDFILE &
		wait
		$CLI -sys $HOST showspace -p -devtype FC  | tail -1 | awk '{print "FCUsable:"$2}'  >> $FCFILE &
		$CLI -sys $HOST showspace -p -devtype NL  | tail -1 | awk '{print "NLUsable:"$2}'  >> $NLFILE &
		$CLI -sys $HOST showspace -p -devtype SSD | tail -1 | awk '{print "SSDUsable:"$2}' >> $SSDFILE &
		wait
		rm $LOCK
	fi
fi

function index
{
	grep -Eh ".:.:. +Data" $SFILE | awk '{print $1}' | awk -F':' '{print $1$2$3}' | sort -u
}

function vvindex
{
	grep -Eh " +t +" $VFILE | head -$(( `grep -Eh " +t +" $VFILE | wc -l ` - 1 )) | awk '{print $1}' | sort -u
}

function num_indexes
{
	index | wc -l
}

function vvnum_indexes
{
	vvindex | wc -l
}

function query
{
	case $VAL in 
        "rio")
		grep -Eh ".:.:. +Data +r" $SFILE | awk '{print $1":"$4}' | awk -F':' '{print $1$2$3":"$4}' | sort
	;;
        "wio")
		grep -Eh ".:.:. +Data +w" $SFILE | awk '{print $1":"$4}' | awk -F':' '{print $1$2$3":"$4}' | sort
	;;
        "rkb")
		grep -Eh ".:.:. +Data +r" $SFILE | awk '{print $1":"$7}' | awk -F':' '{print $1$2$3":"$4}' | sort
	;;
        "wkb")
		grep -Eh ".:.:. +Data +w" $SFILE | awk '{print $1":"$7}' | awk -F':' '{print $1$2$3":"$4}' | sort
	;;
        "rst")
		grep -Eh ".:.:. +Data +r" $SFILE | awk '{print $1":"$10}' | awk -F':' '{print $1$2$3":"$4}' | sort
	;;
        "wst")
		grep -Eh ".:.:. +Data +w" $SFILE | awk '{print $1":"$10}' | awk -F':' '{print $1$2$3":"$4}' | sort
	;;
        "rsz")
		grep -Eh ".:.:. +Data +r" $SFILE | awk '{print $1":"$12}' | awk -F':' '{print $1$2$3":"$4}' | sort
	;;
        "wsz")
		grep -Eh ".:.:. +Data +w" $SFILE | awk '{print $1":"$12}' | awk -F':' '{print $1$2$3":"$4}' | sort
	;;
	"que")
		grep -Eh ".:.:. +Data +t" $SFILE | awk '{print $1":"$14}' | awk -F':' '{print $1$2$3":"$4}' | sort
	;;
	"cpu")
		grep total $CFILE | sed 's/,/ /g' | awk '{print "node"$1"cpu:"100-$5" "}'
	;;
	"location")
		grep -Eh ".:.:. +Data +t" $SFILE | awk '{print $1}' | awk -F':' '{print $1$2$3":"$1"."$2"."$3}' | sort
	;;
	"portType")
		grep -Eh ".:.:. +Data +t" $SFILE | awk '{print $1":"$15}' | awk -F':' '{print $1$2$3":"$4}' | sort
	;;
	*)
		print "Usage: $EXE HOSTNAME query (rio|wio|rkb|wkb|rst|wst|rsz|wsz|que|cpu|location|portType)"
		exit 1
	;;
	esac
}

function vvquery
{
	case $VAL in 
        "rio")
		grep -Eh " +r +" $VFILE | head -`vvnum_indexes` | awk '{print $1":"$3}' | sort
	;;
        "wio")
		grep -Eh " +w +" $VFILE | head -`vvnum_indexes` | awk '{print $1":"$3}' | sort
	;;
        "rkb")
		grep -Eh " +r +" $VFILE | head -`vvnum_indexes` | awk '{print $1":"$6}' | sort
	;;
        "wkb")
		grep -Eh " +w +" $VFILE | head -`vvnum_indexes` | awk '{print $1":"$6}' | sort
	;;
        "rst")
		grep -Eh " +r +" $VFILE | head -`vvnum_indexes` | awk '{print $1":"$9}' | sort
	;;
        "wst")
		grep -Eh " +w +" $VFILE | head -`vvnum_indexes` | awk '{print $1":"$9}' | sort
	;;
        "rsz")
		grep -Eh " +r +" $VFILE | head -`vvnum_indexes` | awk '{print $1":"$11}' | sort
	;;
        "wsz")
		grep -Eh " +w +" $VFILE | head -`vvnum_indexes` | awk '{print $1":"$11}' | sort
	;;
	"que")
		grep -Eh " +t +" $VFILE | head -`vvnum_indexes` | awk '{print $1":"$13}' | sort
	;;
	"vvName")
		grep -Eh " +t +" $VFILE | head -`vvnum_indexes` | awk '{print $1":"$1}' | sort
	;;
	*)
		print "Usage: $EXE HOSTNAME vvquery (rio|wio|rkb|wkb|rst|wst|rsz|wsz|que|vvName)"
		exit 1
	;;
	esac
}

function get
{
	case $VAL in 
        "rio")
		grep -h "$NSPC " $SFILE | egrep " +r +" | awk '{printf "%f",$4}'
	;;
        "wio")
		grep -h "$NSPC " $SFILE | egrep " +w +" | awk '{printf "%f",$4}'
	;;
        "rkb")
		grep -h "$NSPC " $SFILE | egrep " +r +" | awk '{printf "%f",$7}'
	;;
        "wkb")
		grep -h "$NSPC " $SFILE | egrep " +w +" | awk '{printf "%f",$7}'
	;;
        "rst")
		grep -h "$NSPC " $SFILE | egrep " +r +" | awk '{printf "%f",$10}'
	;;
        "wst")
		grep -h "$NSPC " $SFILE | egrep " +w +" | awk '{printf "%f",$10}'
	;;
        "rsz")
		grep -h "$NSPC " $SFILE | egrep " +r +" | awk '{printf "%f",$12}'
	;;
        "wsz")
		grep -h "$NSPC " $SFILE | egrep " +w +" | awk '{printf "%f",$12}'
	;;
	"que")
		grep -h "$NSPC " $SFILE | egrep " +w +" | awk '{printf "%f",$14}'
	;;
	*)
		print "Usage: $EXE HOSTNAME get (rio|wio|rkb|wkb|rst|wst|rsz|wsz|que|cpu) NodeSlotPort"
		exit 1
	;;
	esac
}

function vvget
{
	case $VAL in 
        "rio")
		grep -h "$VOL " $VFILE | egrep " +r +" | awk '{printf "%f",$3}'
	;;
        "wio")
		grep -h "$VOL " $VFILE | egrep " +w +" | awk '{printf "%f",$3}'
	;;
        "rkb")
		grep -h "$VOL " $VFILE | egrep " +r +" | awk '{printf "%f",$6}'
	;;
        "wkb")
		grep -h "$VOL " $VFILE | egrep " +w +" | awk '{printf "%f",$6}'
	;;
        "rst")
		grep -h "$VOL " $VFILE | egrep " +r +" | awk '{printf "%f",$9}'
	;;
        "wst")
		grep -h "$VOL " $VFILE | egrep " +w +" | awk '{printf "%f",$9}'
	;;
        "rsz")
		grep -h "$VOL " $VFILE | egrep " +r +" | awk '{printf "%f",$11}'
	;;
        "wsz")
		grep -h "$VOL " $VFILE | egrep " +w +" | awk '{printf "%f",$11}'
	;;
	"que")
		grep -h "$VOL " $VFILE | egrep " +w +" | awk '{printf "%f",$13}'
	;;
	*)
		print "Usage: $EXE HOSTNAME vvget (rio|wio|rkb|wkb|rst|wst|rsz|wsz|que|cpu) VIRTUAL_VOLUME"
		exit 1
	;;
	esac
}

function freeSpace
{
	if [[ $VAL != "fc" ]] && [[ $VAL != "nl" ]] && [[ $VAL != "ssd" ]] then
		print "Usage: $EXE HOSTNAME freeSpace nl|fc|ssd"
		exit 1
	fi
	cat /tmp/.3par.freespace.$VAL | egrep '(Total|Allocated|Free|Usable)' | tr -d " " | awk -F':' 'BEGIN{ RS = "" }{print "total:"$2/1024000" allocated:"$4/1024000" free:"$6/1024000" usable:"$8/1024000}'
}

case $COM in
	"index")
		index
	;;
	"num_indexes")
		num_indexes
	;;
	"query")
		query
	;;
	"get")
		get
	;;
	"totals")
		totals
	;;
	"diskPorts")
		R=`tail -4 $DFILE | egrep "Data +r" | awk '{print "rio:"$4" rkb:"$6" rst:"$8" rsz:"$10}'`
		W=`tail -4 $DFILE | egrep "Data +w" | awk '{print "wio:"$4" wkb:"$6" wst:"$8" wsz:"$10}'`
		T=`tail -4 $DFILE | egrep "Data +t" | awk '{print "tot:"$1}'`
		print "$R $W $T"
	;;
	"hostPorts")
		R=`tail -4 $HFILE | egrep "Data +r" | awk '{print "rio:"$4" rkb:"$6" rst:"$8" rsz:"$10}'`
		W=`tail -4 $HFILE | egrep "Data +w" | awk '{print "wio:"$4" wkb:"$6" wst:"$8" wsz:"$10}'`
		T=`tail -4 $HFILE | egrep "Data +t" | awk '{print "tot:" $1}'`
		print "$R $W $T"
	;;
	"cpu")
		C=`grep total $CFILE | sed 's/,/ /g' | awk '{printf "node"$1":"100-$5" "}'`
		print $C
	;;
	"cpuIntr")
		C=`grep total $CFILE | sed 's/,/ /g' | awk '{printf "node"$1":"$6" "}'`
		print $C
	;;
	"cpuCtxt")
		C=`grep total $CFILE | sed 's/,/ /g' | awk '{printf "node"$1":"$7" "}'`
		print $C
	;;
	"vvindex")
		vvindex
	;;
	"vvnum_indexes")
		vvnum_indexes
	;;
	"vvquery")
		vvquery
	;;
	"vvget")
		vvget
	;;
	"freeSpace")
		freeSpace
	;;
	*)
		usage
	;;
esac

exit 0
